@@ -77,7 +77,8 @@ def fmt_result(definitions):
     "Format the result of dictionary lookup"
     lines = []
     for defn in definitions:
-        lines.append("<i>" + defn['pos'] + "</i>")
+        if defn['pos'] != "":
+            lines.append("<i>" + defn['pos'] + "</i>")
         lines.extend([str(item[0]+1) + ". " + item[1] for item in list(enumerate(definitions[0]['meaning']))])
     return "<br>".join(lines)
 
@@ -146,7 +147,7 @@ def googledict(word, language, lemmatize=True):
         meanings = []
         for d in item['definitions']:
             meanings.append(d['definition'])
-        meaning_item = {"pos": item['partOfSpeech'], "meaning": meanings}
+        meaning_item = {"pos": item.get('partOfSpeech', ""), "meaning": meanings}
         definitions.append(meaning_item)
     return {"word": word, "definition": definitions}
 
@@ -171,9 +172,8 @@ def lookupin(word, language, lemmatize=True, dictionary="Wiktionary (English)",
         return googletranslate(word, language, gtrans_lang)
     else:
         return {"word": word, "definition": dictdb.define(word, language, dictionary)}
-
-    #print(item)
     return item
+    #print(item)
 
 def getDictsForLang(lang: str):
     "Get the list of dictionaries for a given language"
